! *************************************************************
!   MODULE HVTRANSFC
!   definition of data structure for engine, transmission,
!   body , test cycle, fuel maps
! *************************************************************
module hvtransfc
!   ------------------------------------------------------------
!   TYPE SSPEC
!   vehicle specification
!   ------------------------------------------------------------
    type sspec
        real(8) mua                                             ! air drag coefficient, N/(km/h2)
        real(8) mur                                             ! rolling resistance coeff, N/kg
        real(8) rt                                              ! tire radius, m
        real(8) rt2                                             ! ő^Ca, m
        real(8) tcl                                             ! tire circumference, m
        real(8) bw, bh                                          ! body width, height, m
        real(8) w0                                              ! empty weight, kg
        real(8) wld                                             ! load capacity, kg
        real(8) wt                                              ! test weight, kg
        real(8) gvw                                             ! GVW, kg
        real(8) ric                                             ! highway traveling ratio(0 - 1)
        integer crew                                            ! capacity, persons
        real(8) rld                                             ! Load ratio(0 - 1)
        real(8) rite                                            ! inertia of engine<->t/m in
        real(8) rrc                                             ! Rolling resistance coefficient
        real(8) cd                                              ! air drag coefficient
    end type sspec

!   ------------------------------------------------------------
!   TYPE SENG
!   engine specification
!   ------------------------------------------------------------
    type sengine
        real(8) nex                                             ! max speed, rpm
        real(8) nrate                                           ! rated speed, rpm
        real(8) nidle                                           ! idling speed, rpm
        real(8) nes                                             ! starting speed, rpm
        real(8) nec                                             ! clutch cut speed, rpm
        real(8), pointer :: nel(:)                              ! minimum speed, rpm(for each gear)
    end type sengine

!   ------------------------------------------------------------
!   TYPE SGDAT
!   gear setting ( for each position )
!   ------------------------------------------------------------
    type sgdat
        real(8) gr                                              ! ratio
        real(8) egr                                             ! transmission efficiency
        real(8) tmargin                                             ! torque margin
        real(8) dw                                              ! rotating mass, kg
    end type sgdat

!   ------------------------------------------------------------
!   TYPE stransmission
!   transmission
!   ------------------------------------------------------------
    type stransmission
        integer at                                              ! torque converter AT or not
        integer grs                                             ! starting gear
        integer grb                                             ! bottom gear
        integer skiplimit                                       ! limit of skip shift
        integer ngr                                             ! number of gear (main x sub)
        integer nmgr                                            ! number of gear (main)
        integer nsgr                                            ! number of gear (sub)
        real(8) fgr                                             ! final ratio
        real(8) efgr                                            ! efficiency of final gear
        type (sgdat), pointer :: gri(:)                         ! array of gear setting
    end type stransmission

!   ------------------------------------------------------------
!   TYPE STQCURVE
!   torque curves ( for maximam, frictional )
!   ------------------------------------------------------------
    type stqcurve
        integer ndata                                           ! number of point
        real(8), pointer :: rev(:)                              ! engine speed, rpm
        real(8), pointer :: tq(:)                               ! torque, Nm
    end type stqcurve

!   ------------------------------------------------------------
!   TYPE SCYCLE
!   definition of test cycle for evaluation
!   ------------------------------------------------------------
    type scycle
        integer ndat                                            ! number of test cycle data
        real(8), pointer :: v(:)                                ! test cycle, km/h
        real(8), pointer :: grade(:)                            ! grade, %
    end type scycle

!   ------------------------------------------------------------
!   TYPE SFCMAP
!   fuel map (measured data for input)
!   ------------------------------------------------------------
    type sfcmap
        real(8) fcidle                                          ! idling fuel consumption, L/h
        integer ndata                                           ! number of data point
        real(8), pointer :: rev(:)                              ! engine speed, rpm
        real(8), pointer :: tq(:)                               ! torque, Nm
        real(8), pointer :: fc(:)                               ! fuel consumption, l/h
    end type sfcmap

!   ------------------------------------------------------------
!   TYPE SGRIDMAP
!   grid interpolated fuel map (calculated in the simulation)
!   ------------------------------------------------------------
    type sgridmap
        real(8) fcidle                                          ! idling fuel consumption, l/h
        integer nx                                              ! number of point in X-axis(eng speed)
        integer ny                                              ! number of point in Y-axis(torque)
        real(8), pointer :: rev(:)                              ! engine speed, rpm
        real(8), pointer :: tq(:)                               ! torque, Nm
        real(8), pointer :: fc(:,:)                             ! fuel consumption, l/h
    end type sgridmap

end module hvtransfc



