! **************************************************************
! MAIN PROGRAM convDFC
! **************************************************************
PROGRAM convDFC
!   -----------------------------------------------------------
!   This program uses the GETARG function to acquire the
!   command line option. Please change this part by a suitable
!   method for your fortran90 compiler to use GETARG.
!   ---for Compaq, Intel compiler---
    USE DFLIB   !, ONLY:GETARG
!   ---for Fujitsu compiler---
!   USE SERVICE_ROUTINES, ONLY:GETARG
!   -----------------------------------------------------------

    use modconst
    use hvtransfc
    use testcycle
    use vehicleparams
    use msubfc, only : calcfc, calcave, fcgrid, calcistop
    use dataio
    use mshiftp, only : runmode, maxtqlin
    use dfport   ! 20220907 add.  
    implicit none

    type (sspec) :: spec                                        ! vehicle specification
    type (sengine) :: eng                                       ! engine
    type ( stransmission ) :: tm                                ! transmission
    type ( stqcurve ) :: tqc, ftqc                              ! maximum, frictional torque
    type ( sfcmap ) :: fcm                                      ! measured fuel map
    type ( sgridmap ) :: gfcm                                   ! interpolated grid map
    type ( scycle ) :: vpat, vpat2                              ! test cycle  
    
    character ( len = 1024 ) :: specf='', engf="", tmf=""       ! spec, engine, transmission filename
    character ( len = 1024 ) :: tqf="", ftqf="", mapf=""        ! maximum & frictional torque, map filename
    character ( len = 1024 ) :: outf=''                         ! output name
    character ( len = 1024 ) :: vhname='', tpname=''            ! vehiclename, typename
    character ( len = 1024 ) :: spuamtf="", sphamtf=""          ! AMT_shift filename
    character ( len = 1024 ) :: amtvalf=""                      ! AMT_val.txt filename   ! 200107 add.

    real(8), allocatable :: vu(:), vh(:)                        ! vehicle speed(km/h)
    real(8), allocatable :: neu(:), neh(:), nneu(:), nneh(:)    ! engine speed(rpm), normalized
    real(8), allocatable :: teu(:), teh(:), nteu(:), nteh(:)    ! engine torque(Nm), normalized
    real(8), allocatable :: fcu(:), fch(:)                      ! fuel(L/h)
    integer, allocatable :: spuamt(:),sphamt(:)                 ! AMT_gear position

    integer i, spp, spi, theld
    real(8) vi, vp, eti, evi, verr, maxt
    integer grs                                                 ! start gear
    real(8), pointer :: gr(:), sgr(:)                           ! gear ratio (main, sub)
    real(8) eu, eh, et, ece
    real(8) avvu, avvh, avvt
    integer tsout                                               ! output time series data(1:Yes,0:No)
    real(8) rrc, cd                                             ! running resistance coefficient
    real(8) Kf1, Kf2                                            ! Kf factor
    integer istop, t_istop, istop2, t_istop2, t_istop_off

!   -----------------------------------------------------------
!    Get Specification filename
!   -----------------------------------------------------------
    call GETARG(1, specf)
    call GETARG(2, outf)
    if ( specf == "" ) then
        do while ( specf == '' )
            write( *, '(A,$)' ) ' TYPE FILENAME FOR INPUT : '
            read ( *, * ) specf
        end do
    end if

    if ( outf == "" ) then
        do while ( outf == '' )
            write( *, '(A,$)' ) ' TYPE FILENAME FOR OUTPUT : '
            read ( *, * ) outf
        end do
    end if

!   -----------------------------------------------------------
!   read input data
!   -----------------------------------------------------------
    call readspecfc ( 11, specf, spec%rt, spec%rt2, tm%fgr, tpname, vhname, engf, tmf, tsout, rrc, cd,&
        spuamtf,sphamtf,amtvalf, istop,t_istop, istop2, t_istop2, t_istop_off, Kf1, Kf2 )
    call readgear( 12, tmf, tm%nmgr, gr, tm%nsgr, sgr, grs, tm%at )
    call readengfc( 13, engf, mapf, tqf, ftqf, eng%nidle, eng%nrate, eng%nex )
    call setparameters( tpname, spec, eng, tm, gr, sgr, grs, rrc, cd, amtvalf )
    call readtqfc( 14, trim(tqf), tqc )                         ! maximum torque
    call readtqfc( 15, trim(ftqf), ftqc )                       ! frictional torque
    call readmapfc( 17, trim(mapf), fcm )                       ! fuel map
    call fcgrid( gfcm, fcm, ftqc )                              ! calculate grided fuel map
    call showinputdata( spec, eng, tm, vhname, tpname )         ! display input dataset

!   -----------------------------------------------------------
!   set test cycle for urban
!   -----------------------------------------------------------
    vpat%ndat = nurban
    vpat%v => vurban
    vpat%grade => gurban
 
    allocate ( vu( vpat%ndat ) )                                ! calculated speed
    allocate ( neu( vpat%ndat ) )                               ! engine speed (rpm)
    allocate ( teu( vpat%ndat ) )                               ! engine torque (Nm)
    allocate ( nneu( vpat%ndat ) )                              ! normalized engine speed (%)
    allocate ( nteu( vpat%ndat ) )                              ! normalized engine torque (%)
    allocate ( fcu( vpat%ndat ) )                               ! fuel consumption (l/h)
    allocate ( spuamt( vpat%ndat ) )                            ! shift position of AMT for urban cycle
 
!   --------------------------------------------------------
!   calculate all gear position
!   --------------------------------------------------------
    spp = spuamt(1)                                               
    theld = THOLD
    vp = vpat%v(1)
    verr = 0.0_8
    call readamtshift(600, spuamtf, vpat%ndat, spuamt)
    call runmode ( 1, vpat, vp, spp, theld, THOLD, spec,        &
                   eng, tm, tqc, evi, eti, spi, vi, verr)       ! calculate initial state
    
    do i = 1, vpat%ndat              
        spp = spuamt(i)  
        spi = spuamt(i)
        vp = vi
        verr = 0.0_8

        call runmode ( i, vpat, vp, spp, theld, THOLD, spec,     &
                       eng, tm, tqc, evi, eti, spi, vi, verr )
        maxt = maxtqlin ( tqc, evi )
        vu(i) = vi
        neu(i) = evi
        teu(i) = eti
        nneu(i) = ( neu(i) - eng%nidle )                        &
                   / ( eng%nrate - eng%nidle ) * 100.0_8
        nteu(i) = teu(i) / maxt * 100.0_8
    end do

!   --------------------------------------------------------
!   calculate average fuel consumption
!   --------------------------------------------------------
    call calcfc ( neu, teu, spuamt, fcu, vpat%ndat, ftqc, gfcm, eng )   ! sum-up fuel consumption

    call calcistop( vu, neu, fcu, vpat%ndat, t_istop, t_istop2, t_istop_off)   ! AChXgbvvZ

    call calcave( vu, fcu, vpat%ndat, avvu, eu )                ! average (URBAN)
    call calcave( vu(645:), fcu(645:), 766, avvt, et )          ! average (MID-TOWN)
    
!   -----------------------------------------------------------
!   set test cycle for highway
!   -----------------------------------------------------------
    vpat2%ndat = nhighway
    vpat2%v => vhighway
    vpat2%grade => ghighway

    allocate ( vh  ( vpat2%ndat ) )
    allocate ( neh ( vpat2%ndat ) )
    allocate ( teh ( vpat2%ndat ) )
    allocate ( nneh( vpat2%ndat ) )
    allocate ( nteh( vpat2%ndat ) )
    allocate ( fch ( vpat2%ndat ) )
    allocate ( sphamt( vpat2%ndat ) ) ! shift position of AMT for highway cycle

!   --------------------------------------------------------
!   calculate all gear position
!   --------------------------------------------------------
    call readamtshift(620, sphamtf, vpat2%ndat, sphamt)
    theld = THOLD
    spp = sphamt(1)   
    spi = sphamt(1)    
    vp = vpat2%v(1)
    verr = 0.0_8
    call runmode ( 1, vpat2, vp, spp, theld, THOLD, spec,       &
                   eng, tm, tqc, evi, eti, spi, vi, verr )      ! calculate initial state

    do i = 1, vpat2%ndat
        spp = sphamt(i)  
        spi = sphamt(i)    
        vp = vi
        verr = 0.0_8
        call runmode ( i, vpat2, vp, spp, theld, THOLD, spec,   &
                       eng, tm, tqc, evi, eti, spi, vi, verr )
        maxt = maxtqlin ( tqc, evi )
        vh(i) = vi
        neh(i) = evi
        teh(i) = eti
        nneh(i) = ( neh(i) - eng%nidle )                        &
                    / ( eng%nrate - eng%nidle ) * 100.0_8
        nteh(i) = teh(i) / maxt * 100.0_8
    end do

!   --------------------------------------------------------
!   calculate average fuel consumption
!   --------------------------------------------------------
    call calcfc ( neh, teh, sphamt, fch, vpat2%ndat, ftqc, gfcm, eng )  ! sum-up fuel consumption
    call calcave( vh, fch, vpat2%ndat, avvh, eh )               ! average (INTERCITY) x

    print*, "----------------------------------------------------------"
    if ( tm%at == 1 ) then
        eh = eh * 0.96_8
        eu = eu * 0.91_8
        et = et * 0.91_8
    end if
    
    ece = 1.0_8 / ( ( 1.0_8 - spec%ric ) / eu + spec%ric / eh )

    !print '( 2A,F8.4,A,F0.1 )', " URBAN,    ","FC(km/l) : ",eu, ", Ave.Speed(km/h) : ",avvu
    !print '( 2A,F8.4,A,F0.1 )', " HIGHWAY,  ","FC(km/l) : ",eh, ", Ave.Speed(km/h) : ",avvh
    !print '( 2A,F8.4 )',        " AVERAGE,  ","FC(km/l) : ",ece
    !print *
    !print '( 2A,F8.4,A,F0.1 )', " MID-TOWN, ","FC(km/l) : ",et, ", Ave.Speed(km/h) : ",avvt
    
    
!   -----------------------------------------------------------
!   vZʂ̃`FbNiv_real < v_target, te<100%j  
    call check_results(vpat,vu,neu,teu,nneu,nteu,spuamt,fcu, 'URBAN')
    call check_results(vpat2,vh,neh,teh,nneh,nteh,sphamt,fch, 'HIGHWAY')
!   -----------------------------------------------------------
!    output calculation result
!   -----------------------------------------------------------
!   -----------------------------------------------------------
!    output calculation result
!   -----------------------------------------------------------
    print*
    print '( " OUTPUT FILE : ", A )', trim( outf )
    
    
! o[Wói2022.09.07ǉj-------------------------
    !outf_ver = trim(outf) // "_info.txt"
    open(211, file=trim(outf)//"_info.txt", status = 'unknown' )
    write(211,*) "Version 20221014"
    write(211,*) fdate()
    close(211)
! ------------------------------------------------------------- 

    call outaveragefc ( 21, outf, specf, vhname, tpname, engf, &
                     tmf, tm%fgr, spec%rt, spec%rt2, avvu, eu, avvh, eh, avvt, et, spec%ric, &
                     ece, Kf1, Kf2, istop, istop2, t_istop, t_istop2, t_istop_off)
    if( tsout == 1 ) then
        call outtimeseriesfc (21,outf,vpat,vu,neu,teu,nneu,nteu,spuamt,fcu, 'URBAN')
        call outtimeseriesfc (21,outf,vpat2,vh,neh,teh,nneh,nteh,sphamt,fch, 'HIGHWAY')
    end if

    nullify ( vpat%v, vpat%grade, vpat2%v, vpat2%grade )
    deallocate ( vu, spuamt, neu, teu, nneu, nteu, fcu )
    deallocate ( vh, sphamt, neh, teh, nneh, nteh, fch )
    deallocate ( eng%nel, tm%gri)
    deallocate ( tqc%rev, tqc%tq, ftqc%rev, ftqc%tq )
    deallocate ( fcm%rev, fcm%tq, fcm%fc, gfcm%rev, gfcm%tq, gfcm%fc )  

    print*, ' complete.'

end program convDFC
