
module mshiftp

    interface

        recursive subroutine runmode ( i,vpat,vp,sp,sht,nr,spec,eng,tm,tqc,evi,eti,spi,vi,verr )
            use hvtransfc
            type ( sspec ) :: spec
            type (sengine) :: eng
            type ( stransmission ) :: tm
            type ( stqcurve ) :: tqc
            type ( scycle ) :: vpat
            real(8) vi, evi, eti, vp, verr
            integer nr, i, sp, spi, sht
        end subroutine

        subroutine sftdwn ( i,vpat,vp,sp,spec,eng,tm,tqc,cnt,spout,verr )
            use hvtransfc
            type ( sspec ) :: spec
            type (sengine) :: eng
            type ( stransmission ) :: tm
            type ( stqcurve ) :: tqc
            type ( scycle ) :: vpat
            integer i, sp, spout, cnt
            real(8) vp, verr
        end subroutine

        subroutine sftuptm ( reacc,i,vpat,vp,sp,spec,eng,tm,tqc,cnt,spout,verr )
            use hvtransfc
            type ( sspec ) :: spec
            type (sengine) :: eng
            type ( stransmission ) :: tm
            type ( stqcurve ) :: tqc
            type ( scycle ) :: vpat
            integer i, sp, spout, cnt, reacc
            real(8) vp, verr
        end subroutine

        subroutine sftup ( reacc,i,vpat,vp,sp,spec,eng,tm,tqc,spout,mkt )
            use hvtransfc
            type ( sspec ) :: spec
            type (sengine) :: eng
            type ( stransmission ) :: tm
            type ( stqcurve ) :: tqc
            type ( scycle ) :: vpat
            integer i, sp, spout, reacc, mkt
            real(8) vp
        end subroutine

        integer function startgear ( i, vpat, spec, eng, tm, tqc, stime )
            use hvtransfc
            type ( sspec ) :: spec
            type (sengine) :: eng
            type ( stransmission ) :: tm
            type ( stqcurve ) :: tqc
            type ( scycle ) :: vpat
            integer i, stime
        end function startgear

        subroutine engstat ( ev,et,sp,spec,eng,tm,tqc,vdest,grade,vp,vv,maxt,sw )
            use hvtransfc
            type ( sspec ) :: spec
            type (sengine) :: eng
            type ( stransmission ) :: tm
            type ( stqcurve ) :: tqc
            real(8) ev, et, vdest, maxt, vp, vv, grade
            integer sp, sw
        end subroutine engstat

        function drvfrc ( spec, tm, sp, v1, v2, grade )
            use hvtransfc
            type ( sspec ) :: spec
            type ( stransmission ) :: tm
            integer sp
            real(8) drvfrc, v1, v2, grade
        end function drvfrc

        real(8) function maxtqlin( tqc, rev )
            use hvtransfc
            type (stqcurve) :: tqc
            real(8) rev
        end function maxtqlin

    end interface

end module mshiftp




! *************************************************************
! SUBROUTINE RUNMODE
!  calculation of gear position & vehicle running conditions
!  i : index
!  vpat : test cycle
!  vp : past speed (km/h)
!  sp : past gear position
!  theld : gear holding time (s)
!  nrecur : recursion counter
!  spec : vehicle spec
!  eng : engine data
!  tm : transmission data
!  tqc : maximum torque curve
!  evi : output engine speed
!  eti : output engine torque
!  spi : output gear position
!  vi  : output vehicle speed (km/h)
!  verr : cumulative velocity error
! *************************************************************
recursive subroutine runmode ( i, vpat, vp, sp, theld, nrecur, spec, eng, tm, tqc,     &
                               evi, eti, spi, vi, verr )
    use modconst
    use hvtransfc
    use mshiftp, only : sftup, sftuptm, sftdwn, startgear
    implicit none

    type (scycle) :: vpat
    type (sspec) :: spec
    type (sengine) :: eng
    type (stransmission) :: tm
    type (stqcurve) :: tqc
    integer i, sp, theld, nrecur, spi

    real(8) vp, evi, eti, vi, verr

    real(8) errmin                                              ! minimum error
    integer sw                                                  ! engine condition return code
    integer stime                                               ! starting time length
    real(8) maxt                                                ! maximum engine torque
    real(8) dv                                                  ! velocity difference
    integer reacc                                               ! reacceleration (0:OFF, 1:ON)
    integer spi2, theld2, ct, sw2, s2                           ! for recursion
    real(8) ev2, et2, vi2, verr2
    integer mkt                                                 ! estimated minimum keep time

    vi = vpat%v(i)
 !   evi = eng

    if (vpat%v(i) > 0.0_8 .and. spi==0) then
       write(*,*) 'Vtg[Ŏԑ ', i
       read(*,*) i
       stop
    end if

    if (vpat%v(i) <= 0.0_8) then
       vi = vpat%v(i)
       evi = eng%nidle
       eti = 0.0_8
       spi = 0
       verr = verr &
            + (vpat%v(i-1) + vpat%v(i))/2.0_8 &
            - (vi + vp) /2.0_8
       return
    end if
    
    call engstat ( evi, eti, spi, spec, eng, tm, tqc,           &
                   vpat%v(i), vpat%grade(i), vp, vi, maxt, sw )
    if (sw==1) then
       print *,'Caution : lack of torque, time= ', i
!       stop
    else if(sw==2) then
       print *,'error : engine overrun, time= ', i
       stop   
    else if(sw==3) then
       print *, 'error : engine speed under limit, time= ', i
       stop  
    end if
    
    return
end subroutine runmode



! *************************************************************
! FUNCTION DRVFRC
!  calculate driving force
!  spec : vehicle spec
!  mis  : transmission
!  sp   : gear position
!  v1   : current speed (km/h)
!  v2   : target speed (km/h)
!  grade  : grade (%)
! *************************************************************
real(8) function drvfrc ( spec, tm, sp, v1, v2, grade )
    use hvtransfc
    implicit none

    type ( sspec ) :: spec
    type ( stransmission ) :: tm
    integer sp
    real(8) v1, v2, grade
    real(8) rr, mass, acc, fgrade

    rr =  spec%mur * spec%wt + spec%mua * ( v2 * v2 )
    mass = spec%wt + tm%gri(sp)%dw
    acc = ( v2 - v1 ) / ( 3.6_8 * 1.0_8 )
    fgrade = spec%wt * dsin( datan( grade / 100.0_8 ) ) * 9.8_8

    drvfrc = rr + mass * acc  + fgrade                          ! driving force (N)

end function drvfrc



! *************************************************************
! FUNCTION MAXTQLIN
!  liniear interpolation of torque curve
!  tqc      : torque curve
!  rev      : input engine speed, rpm
!  maxtqlin : interpolated torque, (Nm)
! *************************************************************
real(8) function maxtqlin( tqc, rev )
    use hvtransfc
    implicit none

    type ( stqcurve ) :: tqc                                    ! torque curve array
    real(8) rev                                                 ! input engine speed, rpm
    integer x                                                   ! index

    if ( rev < tqc%rev(1) ) then
        x = 1                                                   ! rev<r[1]
    else if ( rev >= tqc%rev(tqc%ndata) ) then
        x = tqc%ndata -1                                        ! rev>=r[ndata]
    else
        do x = 1, tqc%ndata
            if ( rev >= tqc%rev(x) .and. rev < tqc%rev(x+1) ) then
                exit
            end if
        end do
    end if

    maxtqlin = tqc%tq(x) + ( tqc%tq(x+1)-tqc%tq(x) )            &
             * ( rev-tqc%rev(x) ) / ( tqc%rev(x+1)-tqc%rev(x) )

end function maxtqlin



! *************************************************************
! SUBROUTINE ENGSTAT
!  calculate engine running condition
!  output engine speed, engine torque, maximum torque,
!  vehicle speed, engine condition
!  ev : engine speed
!  et : engine torque
!  sp : gear position
!  spec : vehicle spec
!  eng : engine data
!  tm : transmission data
!  tqc : maximum torque curve
!  vdest : target speed (km/h)
!  grade : grade (%)
!  vp : past speed (km/h)
!  vv : output vehicle speed
!  maxt : maximum torque
!  sw : return code
!   sw=0 engine speed is in range
!   sw=1 lack of torque
!   sw=2 engine overrun
!   sw=3 engine speed under limit
!   sw=4 clutch off
!   sw=5 starting,  Ne <= Nes, shift = 1st or 2nd
! *************************************************************
subroutine engstat ( ev, et, sp, spec, eng, tm, tqc, vdest, grade, vp, vv, maxt, sw )
    use hvtransfc
    use mshiftp, only : maxtqlin, drvfrc
    implicit none

    type ( sspec ) :: spec
    type (stransmission) :: tm
    type (sengine) :: eng
    type ( stqcurve ) :: tqc
    real(8) vp, vdest, vv, grade
    real(8) ev, et, maxt
    integer sp, sw

    real(8) dv, tqdif, df, ds, tgr, tegr
    integer flg, iter

!   ===== CALCULATE INITIAL CONDITION ===================================================
    vv = vdest                                                  ! initialize output speed
    df = drvfrc ( spec, tm, sp, vp, vv, grade )                 ! required driving force
    tgr = tm%gri(sp)%gr * tm%fgr                                ! gear ratio
    tegr = tm%gri(sp)%egr * tm%efgr                             ! efficiency
    if ( df >= 0.0_8 ) then
        et = df * spec%rt / ( tgr * tegr )                      ! engine torque (when driving)
    else
        et = df * ( spec%rt  * tegr ) / tgr                     ! engine torque (when braking)
    end if

    ev = ( vv  * 60.0_8 ) * ( tgr ) / ( spec%tcl * 3.6_8 )      ! engine speed
    dv = vv - vp                                                ! difference of speed

!   -----------------------------------------------------------
!   decelerate
!   -----------------------------------------------------------
    if ( dv < 0.0_8 ) then
        if ( ev < eng%nec ) then                                ! clutch off
            ev = eng%nidle                                       ! idling speed
            et = 0.0_8
            sw = 4                                              ! [ RETURN CODE = 4 ] clutch off
        else
            sw = 0                                              ! [ RETURN CODE = 0 ] normal cond.
        end if
        maxt = maxtqlin ( tqc, ev )
        return
    end if

!   -----------------------------------------------------------
!   accelerate
!   -----------------------------------------------------------
!    if ( ev < eng%nel(sp) .or. vp==0) then               ! engine speed is under lower limit or past vehicle speed = 0
    if ( ev < eng%nel(sp)) then                           ! engine speed is under lower limit
            ev = eng%nes                                        ! starting engine speed  
            maxt = maxtqlin ( tqc, ev )
            if ( et > maxt ) then
                sw = 1                                          ! [ RETURN CODE = 1 ] lack of torque
            else
                sw = 5                                          ! [ RETURN CODE = 5 ] starting
                return
            end if    
    else if (vp==0) then                                        ! past vehicle speed = 0
            if( ev < eng%nes) ev = eng%nes                      ! starting engine speed
            maxt = maxtqlin ( tqc, ev )
            if ( et > maxt ) then
                sw = 1                                          ! [ RETURN CODE = 1 ] lack of torque
            else
                sw = 5                                          ! [ RETURN CODE = 5 ] starting
                return
            end if                    
    
    else if ( ev >= eng%nex ) then                              ! engine speed is over upper limit
        if ( sp == tm%ngr )then                                 ! highest gear
            ev = eng%nex                                        ! engine speed limit
        end if
        vv = ev * spec%tcl * 3.6_8 / ( 60.0_8 * tgr )           ! vehicle speed
        maxt = maxtqlin ( tqc, ev )
        df = drvfrc ( spec, tm, sp, vp, vv, grade )

        if ( df >= 0.0_8 ) then
            et = df * spec%rt / ( tgr * tegr )                  ! engine torque(when driving)
        else
            et = df * ( spec%rt  * tegr ) / tgr                 ! engine torque(when braking)
        end if

        if ( (sp == tm%ngr) .and. ( et > maxt ) .and. ( maxt > 0.0_8 ) ) then
            sw = 1                                              ! [ RETURN CODE = 1 ], lack of torque
        else
            sw = 2                                              ! [ RETURN CODE = 2 ], engine overrun
            return
        end if
     else                                                       ! engine speed is in range
        maxt = maxtqlin ( tqc, ev )                             ! MINIMUM <= ENGINE SPEED < MAXIMUM
        if ( et > maxt .and. maxt > 0.0_8 ) then
            sw = 1                                              ! [ RETURN CODE = 1 ], lack of torque
        else
            sw = 0                                              ! [ RETURN CODE = 0 ], normal cond.
            return
        end if
     end if


!   -----------------------------------------------------------
!   convergence calculation of vehicle speed
!   -----------------------------------------------------------
    ds = 1.0_8
    flg = 0
!    do while ( flg == 0 )
    do iter = 1, 9999
        if(flg /= 0) exit
        df = drvfrc ( spec, tm, sp, vp, vv, grade )

        if ( df >= 0.0_8 ) then
            et = df * spec%rt / ( tgr * tegr )                  ! engine torque(when driving)
        else
            et = df * ( spec%rt  * tegr ) / tgr                 ! engine torque(when braking)
        end if

        ev = vv * 60.0_8 * tgr / ( spec%tcl * 3.6_8 )           ! engine speed

        if ( sp > tm%grb .and. ev < eng%nel(sp) ) then          ! 
!*            if(vp == 0.0) then                                ! 2022.9.01RgAEg
!*                ev = eng%nes                                  !
!*                maxt = maxtqlin ( tqc, ev)                    !
!*            else
!*                maxt = maxtqlin ( tqc, eng%nel(sp) )          ! engine speed is under lower limit
!*            end if
            ev = eng%nel(sp)            !*
            maxt = maxtqlin(tqc, ev)    !*
        else if ( ev < eng%nes ) then
            ev = eng%nes                                        ! starting engine speed
            maxt = maxtqlin ( tqc, ev )
        else
            maxt = maxtqlin ( tqc, ev )
        end if

        tqdif = maxt - et
        if ( ( tqdif < 1.0E-6 ) .and. ( tqdif >= 0.0_8 ) ) then ! check convergence calculation result
            flg = 1
        else if ( tqdif < 0.0_8 ) then
            vv = vv - ds                                        ! decrease target speed
        else
            if(iter==1) then     !* 220901 add. ŏtqdif>0 (maxt > et)̏ꍇɎԑȂ
                flg=1
            else
                ds = ds / 2.0_8                                     ! increase target speed
                vv = vv + ds
            end if
        end if
        if (iter>995) write(*,*)'Warning!: convergence cal. of vehicle speed'
    end do

!   -----------------------------------------------------------
!   final engine running condition
!   -----------------------------------------------------------
    if ( ev < eng%nel(sp) ) then                                ! engine speed is under the lower limit
        vv = 0.0_8                                              ! cannot continue calculation
        maxt = maxtqlin ( tqc, eng%nel(sp) )
        sw = 3                                                  ! [ RETURN CODE = 3 ], out of range
    else if ( ev >= eng%nex ) then                              ! engine overrun
        vv = 0.0_8                                              ! cannot continue calculation
        maxt = maxtqlin ( tqc, eng%nex )
        sw = 2                                                  ! [ return code = 2 ], engine overrun
    end if

end subroutine engstat
